<?php

namespace Drupal\cforge\Access;

use Drupal\user\Entity\User;
use Drupal\user\UserDataInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\KeyValueStore\KeyValueFactoryInterface;
use Symfony\Component\Routing\Route;

/**
 * Determines access to routes based on login status of current user.
 */
class NotNominatedReferrerCheck implements AccessInterface {

  const TIME_LIMIT = '1 month';

  protected $config;
  protected $keyValue;

  function __construct(ConfigFactoryInterface $config_factory, KeyValueFactoryInterface $key_value) {
    $this->config = $config_factory->get('cforge.settings');
    $this->keyValue = $key_value->get('user_referrer');
  }

  /**
   * Grants access to users less than a month old who have not yet nominated a referrer.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The currently logged in account.
   * @param \Symfony\Component\Routing\Route $route
   *   The route to check against.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(AccountInterface $account, Route $route) {
    $referral_fee = $this->config->get('referral_fee');
    if (empty($referral_fee)) {
      return AccessResult::forbidden()->addCacheableDependency($this->config);
    }

    $user = User::load($account->id());
    if ($user->getCreatedTime() > strtotime('-'.SELF::TIME_LIMIT)) {
      $access_result = AccessResult::forbidden('User is older than '.SELF::TIME_LIMIT);
    }
    elseif ($this->keyValue->has($account->id())) {
      $access_result = AccessResult::forbidden('Referral fee already paid for user '.$user->getDisplayName());
    }
    else {
      $access_result = AccessResult::allowed();
    }
    return $access_result->cachePerUser();
  }

}
